/**
 * Copyright 2014 Daum Kakao Corp.
 *
 * Redistribution and modification in source or binary forms are not permitted without specific prior written permission. 
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.kakao.sdk.sample.kakaostory;

import android.app.Activity;
import android.content.DialogInterface;
import android.content.Intent;
import android.os.Bundle;
import android.util.TypedValue;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

import com.kakao.auth.APIErrorResult;
import com.kakao.kakaostory.BasicKakaoStoryPostParamBuilder;
import com.kakao.kakaostory.KakaoStoryCheckUser;
import com.kakao.kakaostory.KakaoStoryHttpResponseHandler;
import com.kakao.kakaostory.KakaoStoryLinkInfo;
import com.kakao.kakaostory.KakaoStoryMyStoriesParamBuilder;
import com.kakao.kakaostory.KakaoStoryMyStoryParamBuilder;
import com.kakao.kakaostory.KakaoStoryProfile;
import com.kakao.kakaostory.KakaoStoryService;
import com.kakao.kakaostory.KakaoStoryService.StoryType;
import com.kakao.kakaostory.LinkKakaoStoryPostParamBuilder;
import com.kakao.kakaostory.MyStoryInfo;
import com.kakao.kakaostory.NoteKakaoStoryPostParamBuilder;
import com.kakao.kakaostory.PhotoKakaoStoryPostParamBuilder;
import com.kakao.sdk.sample.R;
import com.kakao.sdk.sample.common.log.Logger;
import com.kakao.sdk.sample.common.widget.DialogBuilder;
import com.kakao.sdk.sample.common.widget.KakaoToast;
import com.kakao.sdk.sample.common.widget.ProfileLayout;
import com.kakao.usermgmt.LogoutResponseCallback;
import com.kakao.usermgmt.UserManagement;
import com.kakao.usermgmt.UserProfile;
import com.kakao.util.KakaoParameterException;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Locale;

/**
 * 카카오스토리 API인 프로필, 포스팅(이미지 업로드)를 테스트 한다.
 * 유효한 세션이 있다는 검증을 {@link KakaoStoryLoginActivity}로 부터 받고 보여지는 로그인 된 페이지이다.
 */
public class KakaoStoryMainActivity extends Activity implements OnClickListener {
    private final String noteContent = "A Rainbow - William Wordsworth\n" +
                                        "\n" +
                                        "My heart leaps up when I behold\n" +
                                        "A rainbow in the sky:\n" +
                                        "So was it when my life began;\n" +
                                        "So is it now I am a man;\n" +
                                        "So be it when I shall grow old,\n" +
                                        "Or let me die!\n" +
                                        "The Child is father of the Man;\n" +
                                        "I could wish my days to be\n" +
                                        "Bound each to each by natural piety.";

    private final String photoContent = "This cafe is really awesome!";
    private final String linkContent = "better than expected!";
    private final String execParam = "place=1111";
    private final String marketParam = "referrer=kakaostory";
    private String scrapUrl = "http://developers.kakao.com";
    private ProfileLayout profileLayout;
    private UserProfile userProfile;
    private Button getPostButton;
    private Button deletePostButton;
    private Button getPostsButton;
    private String lastMyStoryId;
    private static Activity self;

    /**
     * @param savedInstanceState 기존 session 정보가 저장된 객체
     */
    @Override
    protected void onCreate(final Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        self = KakaoStoryMainActivity.this;
        initializeView();
    }

    private static void redirectLoginActivity(Activity activity) {
        final Intent intent = new Intent(activity, KakaoStoryLoginActivity.class);
        activity.startActivity(intent);
        activity.finish();
    }

    private void onClickIsStoryUser() {
        KakaoStoryService.requestIsStoryUser(new StoryResponseHandler<KakaoStoryCheckUser>() {
            @Override
            public void onHttpSuccess(final KakaoStoryCheckUser kakaoStoryCheckUser) {
                KakaoToast.makeToast(getApplicationContext(), "check story user : " + kakaoStoryCheckUser.isStoryUser(), Toast.LENGTH_SHORT).show();
            }
        });
    }

    private void onClickProfile() {
        KakaoStoryService.requestProfile(new StoryResponseHandler<KakaoStoryProfile>() {
            @Override
            public void onHttpSuccess(final KakaoStoryProfile storyProfile) {
                KakaoToast.makeToast(getApplicationContext(), "succeeded to get story profile", Toast.LENGTH_SHORT).show();
                applyStoryProfileToView(storyProfile);
            }
        });
    }

    private void uploadImage() {
        try {
            final List<File> files = new ArrayList<File>();
//            final File file1 = new File(writeStoryImage(R.drawable.post_image));
//            files.add(file1);
            // gif 예제
            final File file2 = new File(writeStoryImage(R.drawable.kakaostory_animated_gif));
            files.add(file2);
            KakaoStoryService.requestMultiUpload(new StoryResponseHandler<String[]>() {
                @Override
                public void onHttpSuccess(final String[] imageURLs) {
                    try {
                        if (imageURLs != null && imageURLs.length != 0) {
                            requestPostPhoto(imageURLs);
                        } else {
                            KakaoToast.makeToast(getApplicationContext(), "failed to upload image.\nkakaoStoryUpload=null", Toast.LENGTH_SHORT).show();
                        }
                    } finally {
                        deleteTempFiles();
                    }

                }

                @Override
                public void onHttpSessionClosedFailure(final APIErrorResult errorResult) {
                    try {
                        Logger.w("onHttpSessionClosedFailure : %s", errorResult);
                        super.onHttpSessionClosedFailure(errorResult);
                    } finally {
                        deleteTempFiles();
                    }
                }

                @Override
                public void onNotKakaoStoryUser() {
                    try {
                        super.onNotKakaoStoryUser();
                    } finally {
                        deleteTempFiles();
                    }
                }

                @Override
                public void onFailure(final APIErrorResult errorResult) {
                    try {
                        Logger.w("onFailure : %s", errorResult);
                        super.onFailure(errorResult);
                    } finally {
                        deleteTempFiles();
                    }
                }

                private void deleteTempFiles() {
//                    if(file1 != null)
//                        file1.delete();
                    if(file2 != null)
                        file2.delete();
                }

            }, files);
        } catch (Exception e) {
            KakaoToast.makeToast(getApplicationContext(), e.getMessage(), Toast.LENGTH_SHORT).show();
        }
    }

    private void getLinkInfo() {
        KakaoStoryService.requestGetLinkInfo(new StoryResponseHandler<KakaoStoryLinkInfo>() {
            @Override
            public void onHttpSuccess(final KakaoStoryLinkInfo kakaoStoryLinkInfo) {
                if (kakaoStoryLinkInfo != null && kakaoStoryLinkInfo.isValidResult()) {
                    showAlertDialog("succeeded to get link info.\n" + kakaoStoryLinkInfo);
                    requestPostLink(kakaoStoryLinkInfo);
                } else {
                    KakaoToast.makeToast(getApplicationContext(), "failed to get link info.\nkakaoStoryLinkInfo=null", Toast.LENGTH_SHORT).show();
                }
            }
        }, scrapUrl);
    }

    private void onClickLogout() {
        UserManagement.requestLogout(new LogoutResponseCallback() {
            @Override
            public void onSuccess(final long userId) {
                redirectLoginActivity(self);
            }

            @Override
            public void onFailure(final APIErrorResult apiErrorResult) {
                redirectLoginActivity(self);
            }
        });
    }

    private void requestPostNote() {
        final NoteKakaoStoryPostParamBuilder postParamBuilder = new NoteKakaoStoryPostParamBuilder(noteContent);
        requestPost(StoryType.NOTE, postParamBuilder);
    }

    private void requestPostPhoto(final String[] imageURLs) {
        final PhotoKakaoStoryPostParamBuilder postParamBuilder = new PhotoKakaoStoryPostParamBuilder(imageURLs);
        postParamBuilder.setContent(photoContent);
        requestPost(StoryType.PHOTO, postParamBuilder);
    }

    private void requestPostLink(final KakaoStoryLinkInfo kakaoStoryLinkInfo) {
        final LinkKakaoStoryPostParamBuilder postParamBuilder = new LinkKakaoStoryPostParamBuilder(kakaoStoryLinkInfo);
        postParamBuilder.setContent(linkContent);
        requestPost(StoryType.LINK, postParamBuilder);
    }

    private void requestPost(final StoryType storyType, final BasicKakaoStoryPostParamBuilder postParamBuilder) {
        // 앱이 설치되어 있는 경우 kakao<app_key>://kakaostory?place=1111 로 이동.
        // 앱이 설치되어 있지 않은 경우 market://details?id=com.kakao.sample.kakaostory&referrer=kakaostory로 이동
        postParamBuilder.setAndroidExecuteParam(execParam).setIOSExecuteParam(execParam).setAndroidMarketParam(marketParam).setIOSMarketParam(marketParam);
        try {
            final Bundle parameters = postParamBuilder.build();
            KakaoStoryService.requestPost(storyType, new StoryResponseHandler<MyStoryInfo>() {
                @Override
                public void onHttpSuccess(final MyStoryInfo myStoryInfo) {
                    if (myStoryInfo.getId() != null) {
                        lastMyStoryId = myStoryInfo.getId();
                        getPostButton.setEnabled(true);
                        getPostsButton.setEnabled(true);
                        deletePostButton.setEnabled(true);
                        Logger.d("requestPost : %s", "succeeded to post " + storyType + " on KakaoStory.\nmyStoryId=" + lastMyStoryId);
                        KakaoToast.makeToast(getApplicationContext(), "succeeded to post " + storyType + " on KakaoStory.\nmyStoryId=" + lastMyStoryId, Toast.LENGTH_SHORT).show();
                    } else {
                        Logger.d("requestPost : %s", "failed to post " + storyType + " on KakaoStory.\nmyStoryId=null");
                        KakaoToast.makeToast(getApplicationContext(), "failed to post " + storyType + " on KakaoStory.\nmyStoryId=null", Toast.LENGTH_SHORT).show();
                    }
                }
            }, parameters);
        } catch (KakaoParameterException e) {
            Logger.e(e);
            KakaoToast.makeToast(getApplicationContext(), e.getMessage(), Toast.LENGTH_SHORT).show();
        }
    }

    private void requestGetMyStory() {
        try {
            final Bundle parameters = new KakaoStoryMyStoryParamBuilder(lastMyStoryId).build();
            KakaoStoryService.requestGetMyStory(new StoryResponseHandler<MyStoryInfo>() {
                @Override
                public void onHttpSuccess(final MyStoryInfo myStoryInfo) {
                    String message;
                    if (lastMyStoryId.equals(myStoryInfo.getId())) {
                        Logger.d("succeeded to get my post from KakaoStory.\nmyStoryInfo=" + myStoryInfo);
                        message = "succeeded to get my post from KakaoStory.\nmyStoryInfo=" + myStoryInfo;
                    } else {
                        Logger.d("failed to get my post from KakaoStory.\nexpectedId=" + lastMyStoryId + ",id=" + myStoryInfo.getId());
                        message = "failed to get my post from KakaoStory.\nexpectedId=" + lastMyStoryId + ",id=" + myStoryInfo.getId();
                    }

                    showAlertDialog(message);
                }
            }, parameters);
        } catch (KakaoParameterException e) {
            Logger.e(e);
            KakaoToast.makeToast(getApplicationContext(), e.getMessage(), Toast.LENGTH_SHORT).show();
        }
    }

    private void showAlertDialog(String message) {
        new DialogBuilder(KakaoStoryMainActivity.this)
                .setMessage(message)
                .setPositiveButton(android.R.string.ok, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        dialog.dismiss();
                    }
                }).create().show();
    }

    private void requestGetMyStories() {
        final Bundle parameters = new KakaoStoryMyStoriesParamBuilder(lastMyStoryId).build();
        KakaoStoryService.requestGetMyStories(new StoryResponseHandler<MyStoryInfo[]>() {
            @Override
            public void onHttpSuccess(final MyStoryInfo[] myStories) {
                String message = "succeeded to get my posts from KakaoStory." +
                        "\ncount=" + myStories.length +
                        "\nstories=" + Arrays.toString(myStories);
                showAlertDialog(message);
            }
        }, parameters);
    }

    private void requestDeleteMyStory() {
        try {
            final Bundle parameters = new KakaoStoryMyStoryParamBuilder(lastMyStoryId).build();
            KakaoStoryService.requestDeleteMyStory(new StoryResponseHandler<Void>() {

                @Override
                public void onHttpSuccess(Void resultObj) {
                    lastMyStoryId = null;
                    getPostButton.setEnabled(false);
                    getPostsButton.setEnabled(false);
                    deletePostButton.setEnabled(false);
                    KakaoToast.makeToast(getApplicationContext(), "succeeded to delete my post from KakaoStory.\nid=" + lastMyStoryId, Toast.LENGTH_SHORT).show();
                }
            }, parameters);
        } catch (KakaoParameterException e) {
            KakaoToast.makeToast(getApplicationContext(), e.getMessage(), Toast.LENGTH_SHORT).show();
        }
    }

    private void initializeView() {
        setContentView(R.layout.layout_kakaostory_main);
        profileLayout = (ProfileLayout) findViewById(R.id.com_kakao_user_profile);
        profileLayout.setDefaultBgImage(R.drawable.bg_image_01);
        profileLayout.setDefaultProfileImage(R.drawable.thumb_story);
        ((TextView)findViewById(R.id.text_title)).setText(getString(R.string.text_kakaostory));

        findViewById(R.id.title_back).setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                finish();
            }
        });

        // 로그인 하면서 caching되어 있는 profile를 그린다.
        userProfile = UserProfile.loadFromCache();
        if (userProfile != null) {
            profileLayout.setUserProfile(userProfile);
        }

        initializeButtons();
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.user_check_button:
                onClickIsStoryUser();
                break;
            case R.id.profile_button:
                onClickProfile();
                break;
            case R.id.text_post_button:
                requestPostNote();
                break;
            case R.id.image_post_button:
                uploadImage();
                break;
            case R.id.link_post_button:
                getLinkInfo();
                break;
            case R.id.get_post_button:
                if(lastMyStoryId == null){
                    KakaoToast.makeToast(getApplicationContext(), "post first then get info", Toast.LENGTH_SHORT).show();
                } else {
                    requestGetMyStory();
                }
                break;
            case R.id.delete_post_button:
                if(lastMyStoryId == null){
                    KakaoToast.makeToast(getApplicationContext(), "post first then delete it", Toast.LENGTH_SHORT).show();
                } else {
                    requestDeleteMyStory();
                }
                break;
            case R.id.get_posts_button:
                if (lastMyStoryId == null) {
                    KakaoToast.makeToast(getApplicationContext(), "post first then get info", Toast.LENGTH_SHORT).show();
                } else {
                    requestGetMyStories();
                }
                break;
            case R.id.logout_button:
                onClickLogout();
                break;
        }
    }

    private void initializeButtons() {
        findViewById(R.id.user_check_button).setOnClickListener(this);
        findViewById(R.id.profile_button).setOnClickListener(this);
        findViewById(R.id.text_post_button).setOnClickListener(this);
        findViewById(R.id.image_post_button).setOnClickListener(this);
        findViewById(R.id.link_post_button).setOnClickListener(this);
        findViewById(R.id.logout_button).setOnClickListener(this);

        // 마지막 포스팅 정보.
        getPostButton = (Button) findViewById(R.id.get_post_button);
        getPostButton.setOnClickListener(this);

        // 마지막 포스팅 삭제
        deletePostButton = (Button) findViewById(R.id.delete_post_button);
        deletePostButton.setOnClickListener(this);

        // 최근 포스팅 정보.
        getPostsButton = (Button) findViewById(R.id.get_posts_button);
        getPostsButton.setOnClickListener(this);
    }

    // profile view에서 story profile을 update 한다.
    private void applyStoryProfileToView(final KakaoStoryProfile storyProfile) {
        if (profileLayout != null) {
            if (userProfile != null) {
                profileLayout.setUserProfile(userProfile);
            }

            final String nickName = storyProfile.getNickName();
            if (nickName != null) {
                profileLayout.setNickname(nickName);
            }

            final String profileImageURL = storyProfile.getProfileImageURL();
            if (profileImageURL != null) {
                profileLayout.setProfileURL(profileImageURL);
            }

            final String birthDay = storyProfile.getBirthday();
            if (birthDay != null) {
                profileLayout.setBirthDay(birthDay);
            }

            final String backgroundURL = storyProfile.getBgImageURL();
            if (backgroundURL != null) {
                profileLayout.setBgImageURL(backgroundURL);
            }
        }
    }

    private String writeStoryImage(final int imageResourceId) throws IOException {
        final int bufferSize = 1024;
        InputStream ins = null;
        FileOutputStream fos = null;
        String outputFileName = null;
        try {
            ins = getResources().openRawResource(imageResourceId);

            final TypedValue value = new TypedValue();
            getResources().getValue(imageResourceId, value, true);
            final String imageFileName =  value.string == null ? null : value.string.toString();
            String extension = null;
            if(imageFileName != null)
                extension = getExtension(imageFileName);
            if(extension == null)
                extension = ".jpg";

            final File diskCacheDir = new File(getCacheDir(), "story");

            if (!diskCacheDir.exists())
                diskCacheDir.mkdirs();

            outputFileName = diskCacheDir.getAbsolutePath() + File.separator + "temp_" + System.currentTimeMillis() + extension;

            fos = new FileOutputStream(outputFileName);

            byte[] buffer = new byte[bufferSize];
            int read;
            while((read = ins.read(buffer)) != -1){
                fos.write(buffer, 0, read);
            }
        } finally {
            try {
                if (ins != null)
                    ins.close();
                if (fos != null)
                    fos.close();
            } catch (Throwable t) {
            }
        }

        return outputFileName;
    }

    public static String getExtension(String fileName) {
        String ext = null;
        int i = fileName.lastIndexOf('.');
        int endIndex = fileName.lastIndexOf("?");

        if (i > 0 && i < fileName.length() - 1) {
            Locale curLocale = Locale.getDefault();
            if (endIndex < 0) {
                ext = fileName.substring(i).toLowerCase(curLocale);
            } else {
                ext = fileName.substring(i, endIndex).toLowerCase(curLocale);
            }
        }
        return ext;
    }

    private static class StoryResponseHandler<T> extends KakaoStoryHttpResponseHandler<T> {
        @Override
        public void onHttpSuccess(T resultObj) {
            redirectLoginActivity(self);
        }

        @Override
        public void onHttpSessionClosedFailure(final APIErrorResult errorResult) {
            redirectLoginActivity(self);
        }

        @Override
        public void onNotKakaoStoryUser() {
            KakaoToast.makeToast(self, "not KakaoStory user", Toast.LENGTH_SHORT).show();
        }

        @Override
        public void onFailure(final APIErrorResult errorResult) {
            final String message = "MyKakaoStoryHttpResponseHandler : failure : " + errorResult;
            Logger.w(message);
            KakaoToast.makeToast(self, message, Toast.LENGTH_LONG).show();
        }
    }
}